/*
 * gc global variable provides access to GUI Composer infrastructure components and project information.
 * For more information, please see the Working with Javascript guide in the online help.
 */
var gc = gc || {};
gc.services = gc.services || {};
var saveAsFilePath; // undefined initially
var checkPollingTimeout = null;

/*
 *  Global variables for Speed Profile Graphing
 */
var linear_duty;
var linear_speed;
var staircase_duty;
var staircase_speed;
var forward_reverse_duty;
var forward_reverse_speed;
var linear_and_stair_layout;
var forward_reverse_layout;
var speed_profile_params;

/*
 *  Boilerplate code for creating computed data bindings
 */
document.addEventListener('gc-databind-ready', function () {

  var onDeviceConnectionChange = new gc.databind.IChangedListener();

  var getDeviceObj = function () {
    var deferred = Q.defer();

    setTimeout(function () {
      gc.target.access.readValue('gDRV8889_Obj').then(function (value) {
        deferred.resolve(value);
      }).fail(function (reason) {
        deferred.reject(reason);
      });
    }, 250);

    return deferred.promise;
  }

  var updateDeviceConnectionState = function (oldState, newState) {
    if (newState) {
      updateDeviceConnectionStateInallComponents(true, 'MCF_GUI');
    } else {
      updateDeviceConnectionStateInallComponents(false);
    }
  }

  var updateDeviceConnectionStateInallComponents = function(state, deviceID) {
    if (state) {
      templateObj.$.ti_widget_eventlog_view.log(
        "info",
        "DRV" + deviceID + "-Q1 EVM connected"
      );
      clearTimeout(checkPollingTimeout);
      checkPollingTimeout = setTimeout(checkPolling, 500);
    } else {
      gc.connectionManager.disconnect();
      clearTimeout(checkPollingTimeout);
      templateObj.$.ti_widget_eventlog_view.log("info", "EVM disconnected");
    }
  };

  var checkPolling = function() {
    if (commQueue.length === 0) {
      getIntGlobalVariable("MEMADDR_PROG").then(() => {
        clearTimeout(checkPollingTimeout);
        checkPollingTimeout = setTimeout(checkPolling, 500);
      }).fail((error)=>{
        console.log(error);
      })
    } else {
      clearTimeout(checkPollingTimeout);
      checkPollingTimeout = setTimeout(checkPolling, 500);
    }
  };

  //   gc.autoDetectPortsFilter = function(transport, ports, info) {
  //     return ports.filter(function(port) {
  //       return port.manufacturer === "FTDI";
  //     });
  //   }

  onDeviceConnectionChange.onValueChanged = updateDeviceConnectionState.bind(this);
  gc.databind.registry.getBinding('my_model.$target_connected').addChangedListener(onDeviceConnectionChange);

  // hide the default Program Device Menu
  let programDeviceMenu = document.querySelector('#FileProgram');
  let menuAction = programDeviceMenu.querySelector('#menuAction');
  menuAction.style.display="none";
  /*
   *   Add custom computed value databindings here, using the following method:
   *
   *   function gc.databind.registry.bind(targetBinding, modelBinding, [getter], [setter]);
   *
   *   param targetBinding - single binding string or expression, or array of binding strings for multi-way binding.
   *   param modelBinding - single binding string or expression, or array of binding strings for multi-way binding.
   *   param getter - (optional) - custom getter function for computing the targetBinding value(s) based on modelBinding value(s).
   *   param setter - (optional) - custom setter function for computing the modelBinding value(s) based on targetBinding value(s).
   */
  /* Computed Data Bindings - BEGIN */


  /* Computed Data Bindings - END */


  // For example, a simple computed values based on simple expression
  // gc.databind.registry.bind('widget.id.propertyName', "targetVariable == 1 ? 'binding is one' : 'binding is not one'");

  // Or a custom two-way binding with custome getter and setter functions.  (setter is optional)  (getter only indicates one-way binding)
  // gc.databind.registry.bind('widget.id.propertyName', "targetVariable", function(value) { return value*5/9 + 32; }, function(value) { (return value-32)*9/5; });

  regBindings();
  regPropsVariablesBindings();
  // Event 1 to n bindings
  /*
    gc.databind.registry.bind('widget.date.value',
        // dependant bindings needed in order to compute the date, in name/value pairs.
        {
            weekday: 'widget.dayOfWeek.selectedText',
            day: 'widget.dayOfMonth.value',
            month: 'widget.month.selectedText',
            year: 'widget.year.value'
        },
        // getter for date computation
        function(values)
        {
            // compute and return the string value to bind to the widget with id 'date'
            return values.weekday + ', ' + values.month + ' ' + values.day + ', ' + values.year;
        }
    );
	*/
    
    // Initialize speed profile parameters and draw the plot
    // initializeSpeedProfilePlot();
    
});

/*
 *  Boilerplate code for creating custom actions
 */
document.addEventListener('gc-nav-ready', function () {
  /*
   *   Add custom actions for menu items using the following api:
   *
   *   function gc.nav.registryAction(id, runable, [isAvailable], [isVisible]);
   *
   *   param id - uniquely identifies the action, and should correspond to the action property of the menuaction widget.
   *   param runable - function that performs the custom action.
   *   param isAvailable - (optional) - function called when the menu action is about to appear.  Return false to disable the action, or true to enable it.
   *   param isVisible - (optional) - function called when the menu action is about to appear.  Return false to hide the action, or true to make it visible.
   */

  // For example,
  // gc.nav.registerAction('myCustomCloseAction', function() { window.close(); }, function() { return true; }, function() { return true; });

  // Alternatively, to programmatically disable a menu action at any time use:
  // gc.nav.disableAction('myCustomCloseAction);    then enable it again using:  gc.nav.enableAction('myCustomCloseAction');

  gc.nav.registerAction('show_introduction', function () {
    if ((templateObj) && (templateObj.$)) {
      templateObj.$.ti_widget_vtabcontainer.selectedIndex = 0;
      templateObj.$.ti_widget_menuitem.hideFullMenu();
    }
  }, function () {
    return true;
  }, function () {
    return true;
  });
  gc.nav.registerAction('show_registers', function () {
    if ((templateObj) && (templateObj.$)) {
      templateObj.$.ti_widget_vtabcontainer.selectedIndex = 1;
      templateObj.$.ti_widget_menuitem.hideFullMenu();
    }
  }, function () {
    return true;
  }, function () {
    return true;
  });
  gc.nav.registerAction('show_motor_control', function () {
    if ((templateObj) && (templateObj.$)) {
      templateObj.$.ti_widget_vtabcontainer.selectedIndex = 2;
      templateObj.$.ti_widget_menuitem.hideFullMenu();
    }
  }, function () {
    return true;
  }, function () {
    return true;
  });

  gc.nav.registerAction('load_registers', function () {
    if ((templateObj) && (templateObj.$)) {
      gc.File.browseAndLoad(null, {
        bin: false
      }, function (contents, properties) {
        var regData = null;
        try {
          regData = contents;
          if (regData) {
            templateObj.setRegisterData(regData);
            templateObj.$.ti_widget_eventlog_view.log("info", "Load Registers: loaded from :" + properties.localPath);
          }
        } catch (ex) {
          templateObj.logMsgAndNotify("error", "Load Registers: Could not parse file:" + ex, false, true);
        }

      });
    }
  });

  gc.nav.registerAction('save_registers', function () {
    if ((templateObj) && (templateObj.$)) {
      var regData = templateObj.getRegisterData();
      let blob = new Blob([JSON.stringify(JSON.parse(regData), null, 2)], {
        type: "application/json;charset=utf-8"
      });
      saveAs(blob, "regs.json");
    }
  });

  gc.nav.registerAction('open_log_pane', function () {
    if ((templateObj) && (templateObj.$)) {
      templateObj.$.ti_widget_eventlog_view.openView();
    }
  }, function () {
    return true;
  }, function () {
    return true;
  });

  gc.nav.registerAction('open_scripting_window', function () {
    window.open('app/scripting.html', '_evm_scripting');
  }, function () {
    return true;
  }, function () {
    return true;
  });
});

/*
 *  Boilerplate code for working with components in the application gist
 */


var initComplete = false;
var templateObj;
var registerModel;

// Wait for DOMContentLoaded event before trying to access the application template
var init = function () {
  templateObj = document.querySelector('#template_obj');

  // Wait for the template to fire a dom-change event to indicate that it has been 'stamped'
  // before trying to access components in the application.
  templateObj.addEventListener('dom-change', function () {
    if (initComplete) return;
    this.async(function () {
      initComplete = true;
      console.log("Application template has been stamped.");
      templateObj.$.ti_widget_toast.hideToast();
      templateObj.$.ti_widget_eventlog_view.log("info", "Application started.");
      registerModel = Polymer.dom(document).querySelector('ti-model-register');
      if (registerModel) {
        registerModel.addEventListener("usbHidModelReceivingData", function (event) {
          templateObj.logMsgAndNotify("debug", "Receiving data from target.", false, false);
        });
      } else {
        console.log("no ti-model-register component found in this application!")
      }

      // Now that the template has been stamped, you can use 'automatic node finding' $ syntax to access widgets.
      // e.g. to access a widget with an id of 'widget_id' you can use templateObj.$.widgetId

      
      if (templateObj.$.ti_widget_multidevice_app_mgr) {
        const boardNames = templateObj.$.ti_widget_multidevice_app_mgr.boardNames;
        if (boardNames.indexOf(',') === -1) {
          templateObj.$.ti_widget_multidevice_app_mgr.selectedBoard = boardNames;
        }
      }

      // When the user clicks the back button, this event is fired which will reset the app and redirects the app to landing page
      templateObj.$.ti_widget_vtabcontainer.addEventListener('app_reset', function (event) {
        if (gc.connectionManager) {
          gc.connectionManager.disconnect();
        }
        templateObj.$.ti_widget_multidevice_app_mgr.resetApp();
      });


      if (templateObj.$.ti_widget_tabcontainer_guide) {
        templateObj.$.ti_widget_tabcontainer_guide.selectedIndex = 0; // always start with the home page

        templateObj.$.ti_widget_label_expert.addEventListener("click", function (event) {
          templateObj.$.ti_widget_vtabcontainer.selectedIndex = 2;
        });

        templateObj.$.ti_widget_label_beginner.addEventListener("click", function (event) {
          templateObj.$.ti_widget_tabcontainer_guide.selectedIndex = 1;
        });

        templateObj.skipPage = function () {
          templateObj.$.ti_widget_tabcontainer_guide.selectedIndex = 1 + templateObj.$.ti_widget_tabcontainer_guide.selectedIndex;
        };

        templateObj.$.ti_widget_label_skip_page.addEventListener("click", templateObj.skipPage);
      }

      templateObj.$.ti_widget_vtabcontainer.addEventListener("selected_tab_changed", function (event) {
        if (event.detail.selectedIndex === 0) {
          // When there is only one device in a multidevice app, select the first device automatically
          if (templateObj.$.ti_widget_multidevice_app_mgr) {
            const boardNames = templateObj.$.ti_widget_multidevice_app_mgr.boardNames;
            if (boardNames.indexOf(',') === -1) {
              templateObj.$.ti_widget_multidevice_app_mgr.selectedBoard = boardNames;
            }
          }
        }
        let graphComponents = templateObj.$.ti_widget_vtabcontainer.querySelectorAll('ti-mdbu-custom-graph-widget');
        for (let graphComponent of graphComponents) {
          graphComponent._updateLayoutDimension();
        }
      });

      // open the eventlog viewer when the user clicks on the book icon in the status bar:
      templateObj.$.ti_widget_eventlog_view.closeView(); // ensure that view states are in sync with application
      templateObj.$.ti_widget_statusbar.addEventListener("appiconclicked", function (event) {
        templateObj.$.ti_widget_eventlog_view.toggleView();
      });


      if (templateObj.$.ti_widget_register_page) {
        //hide bit field section
        templateObj.$.ti_widget_register_page.querySelector('ti-widget-registerview').isBitDisplayVisible = false;

        // get the active device name from the register page
        templateObj.$.ti_widget_menubar.productName = templateObj.$.ti_widget_register_page.deviceName;

        // called when the user clicks the Write Register button:
        templateObj.$.ti_widget_register_page.addEventListener("write_reg", function (event) {
          var detailsObj = event.detail;
          templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
        }.bind(this));

        // called when the user clicks the Write All Registers button:
        templateObj.$.ti_widget_register_page.addEventListener("write_regs", function (event) {
          var detailsObj = event.detail;
          templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
        }.bind(this));

        // called when the user clicks the Read Register button:
        templateObj.$.ti_widget_register_page.addEventListener("read_reg", function (event) {
          var detailsObj = event.detail;
          templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
        }.bind(this));

        // called when the user clicks the Read All Registers button:
        templateObj.$.ti_widget_register_page.addEventListener("read_regs", function (event) {
          var detailsObj = event.detail;
          templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
        }.bind(this));

        // called when the user changes the immediate / deferred mode
        templateObj.$.ti_widget_register_page.addEventListener("update_mode_changed", function (event) {
          var detailsObj = event.detail;
          templateObj.$.ti_widget_eventlog_view.log("info", "request to change update mode to " + detailsObj.mode.label);
        }.bind(this));

        // called when the user changes the auto read refresh rate
        templateObj.$.ti_widget_register_page.addEventListener("auto_read_config_changed", function (event) {
          var detailsObj = event.detail;
          templateObj.$.ti_widget_eventlog_view.log("info", "request to change automatic read interval to " + detailsObj.autoRead.label);
        }.bind(this));
      }

      var usbHidService = gc.services['ti-service-usbhid'];
      if (usbHidService) {
        usbHidService.addEventListener('usbHidError', function (event) {
          var detailsObj = event.detail;
          templateObj.logMsgAndNotify("error", detailsObj.info + ": " + detailsObj.error, false, true);
        })
      }

      templateObj.getRegisterData = function () {
        var regData = {
          signature: "oneui-register-data",
          data: []
        }
        var regBlocks = templateObj.$.ti_widget_register_page.regBlocks;
        for (var i = 0; i < regBlocks.length; i++) {
          var registersData = [];
          for (var r = 0; r < regBlocks[i].registers.length; r++) {
            var reg = regBlocks[i].registers[r];
            var regName = reg.name.toLowerCase();
            regName = regName.replace(/\&/g, "_");
            var item = {
              idx: r,
              id: regName,
              value: reg.value
            }
            registersData.push(item);
          }
          regData.data.push(registersData);
        }
        var strJson = JSON.stringify(regData);
        return strJson;
      };

      templateObj.setRegisterData = function (strJson) {
        var regData = JSON.parse(strJson);
        var ok = true;
        var regBlocks = templateObj.$.ti_widget_register_page._registerView.regBlocks;
        for (var b = 0; b < regBlocks.length; b++) {
          var registersData = [];
          for (var r = 0; r < regBlocks[b].registers.length; r++) {
            var reg = regBlocks[b].registers[r];
            var regName = reg.name.toLowerCase();
            regName = regName.replace(/\&/g, "_");
            item = regData.data[b][r];
            if (item.id === regName) {
              reg.value = item.value;
              if (!templateObj.$.ti_widget_register_page.isDeferredWrite) {
                reg.valueLastWritten = reg.value;
                var intValue = gc.utils.getValueFromHexString("" + reg.value);
                templateObj.$.ti_widget_register_page.writeValueToTargetRegister(intValue, null, b, r);
              } else {
                templateObj.$.ti_widget_register_page._registerView.refreshRegValue(b, r);
              }
            } else {
              var msg = "load registers failed: register name mismatch (" + item.id + " does not match " + regName + ")";
              templateObj.logMsgAndNotify("warning", msg, false, true);
            }
          }
        }
        if (templateObj.$.ti_widget_register_page.isDeferredWrite) {
          templateObj.$.ti_widget_register_page._registerView._deferredAccessCtr++; // update the page
        }
      };

      templateObj.showToastMsg = function (msgType, msg, durationInSeconds) {
        if (!msg || this.readingContinuously) {
          return;
        }
        if (gc.connectionManager.status != "connected") {
          msgType = "error";
          msg = "Must be connected to target device to read registers."
        }
        templateObj.$.ti_widget_toast.hideToast();
        templateObj.$.ti_widget_toast.message = msg;
        switch (msgType.toLowerCase()) {
          case "error":
            templateObj.$.ti_widget_toast.iconName = "error";
            templateObj.$.ti_widget_toast.backgroundColor = "red";
            break;
          case "warning":
            templateObj.$.ti_widget_toast.iconName = "warning";
            templateObj.$.ti_widget_toast.backgroundColor = "orange";
            break;
          default:
            templateObj.$.ti_widget_toast.iconName = "check";
            templateObj.$.ti_widget_toast.backgroundColor = "teal";
            break;
        }
        templateObj.$.ti_widget_toast.duration = +durationInSeconds;
        templateObj.$.ti_widget_toast.showToast();
        console.log("toast msg [" + msgType + "]: " + msg);
      };

      templateObj.logMsgAndNotify = function (msgType, msg, showInStatusBar, showInToast) {
        var iconName = templateObj.$.ti_widget_eventlog_view.iconForItem(msgType);
        if (showInStatusBar) {
          var statusBar = gc.services['ti-widget-statusbar'];
          if (statusBar) {
            statusBar.statusString3 = msg;
            statusBar.setIcon(iconName);
          }
        }
        if (showInToast) {
          templateObj.showToastMsg(msgType, msg, 4);
        }
        templateObj.$.ti_widget_eventlog_view.log(msgType, msg);
      };

      templateObj.refreshRegmap = function () {
        var temp = this.$.ti_widget_register_page.selectedRegister;
        this.$.ti_widget_register_page.selectedRegister = temp.parentGroup.registers[0];
        this.$.ti_widget_register_page.selectedRegister = temp.parentGroup.registers[1];
        this.$.ti_widget_register_page.selectedRegister = temp;

      }

      templateObj.getRegField = function (modelName, registerName, fieldName) {
        var regBlocks = this.$.ti_widget_register_page.regBlocks;
        for (var block of regBlocks) {
          for (var register of block.registers) {
            if (register.name === registerName) {
              if (fieldName) {
                for (var field of register.fields) {
                  if (field.name === fieldName) {
                    return field;
                  }
                }
              }
              return register;
            }
          }
        }
      }

      templateObj.getHighlevelBindedControls = function (modelName, registerName, fieldName) {
        var regPath = modelName + '.' + registerName;
        if (fieldName) {
          regPath = regPath + '.' + fieldName;
        }
        var result = {};
        result.destBindControls = [];
        result.srcBindRegField = this.getRegField(modelName, registerName, fieldName);
        var bindObj = gc.databind.registry.getBinding(regPath);
        for (var item of bindObj.fEvents.fListeners.StatusChanged) {
          result.destBindControls.push(item.destBinding.widgetId);
        }
        return result;
      }

      function showIncompleteSteps(stepListComponent, proceedCallback) {
        var stepTabContainer = stepListComponent.querySelector("ti-mdbu-custom-step-list-tab-container");
        var incompSteps = stepTabContainer._getIncompleteSteps();
        if (incompSteps.length !== 0) {
          var steps = [];
          incompSteps.forEach(function (stepTab) {
            steps.push(stepTab.stepSummary);
          });
          var infoObj = {
            title: "Proceed to next Step?",
            heading: "Make sure you have completed the following",
            steps: steps
          };
          templateObj.$.ti_mdbu_custom_info_dialog.openInfoDialog(infoObj, proceedCallback);
        } else {
          proceedCallback();
        }
      }

      if (templateObj.$.ti_widget_button_connect) {
        templateObj.$.ti_widget_button_connect.addEventListener("click", function (event) {
          if (gc.connectionManager) {
            gc.connectionManager.connect();
          }
        });
      }

      // Initialize Floating-info container, Draggable container and right_side panel container
      templateObj.$.ti_mdbu_custom_floating_info.hidden = true;
      templateObj.$.ti_mdbu_draggable_container.hidden = true;
      templateObj.$.ti_right_panel_widget_container18.hidden = true;

      gc.fileCache.readJsonFile('app/eventBindings.json').then((content) => {
        const eventBindings = content.eventBindings;
        eventBindings.forEach(binding => {
          // const bindingSrc = document.querySelector('#' + binding.eventEmitter);
          const bindingSrc = templateObj.$[binding.eventEmitter];
          bindingSrc.addEventListener(binding.eventName, function (e) {
            // const eventTarget = templateObj.$[binding.eventTarget];
            // if(binding.eventParams) {
            //   eventTarget[binding.eventTriggerMethod](e, ...binding.eventParams);
            // } else {
            //   eventTarget[binding.eventTriggerMethod](e);
            // }
            window[binding.eventHandler].bind(templateObj, e)();
          });
        });
      });

      document.addEventListener('status-bar-update', (evt) => {
        let statusBar = templateObj.$.ti_widget_statusbar;
        statusBar.progress = evt.detail.progress;
        statusBar.progressBarEnabled = true;
        statusBar.statusString3 = evt.detail.message;
        if (evt.detail.showToast) {
          templateObj.showToastMsg(evt.detail.error ? 'error' : 'success', evt.detail.message, 4);
        }
      });

      gc.autoDetectPortsFilter = function (transport, ports, info) {
        return ports.filter(function (port) {
          return port.manufacturer === "FTDI";
        });
      }
      
      templateObj.$.ti_mdbu_custom_graph_widget._selectedBoardChanged();

        // var hcframeworkjs = document.createElement("script");
        // hcframeworkjs.type = "text/javascript";
        // hcframeworkjs.src = "https://code.highcharts.com/adapters/standalone-framework.js";
        
        // hcframeworkjs.onload = function() {
        //     var highchartsjs = document.createElement("script");
        //     highchartsjs.type = "text/javascript";
        //     highchartsjs.src = "https://code.highcharts.com/highcharts.js";
            
        //     highchartsjs.onload = function() {
        //         var controlplotjs = document.createElement("script");
        //         controlplotjs.type = "text/javascript";
        //         controlplotjs.src = "app/bode/controlplot.js";
                
        //         controlplotjs.onload = function() {
        //             var demojs = document.createElement("script");
        //             demojs.type = "text/javascript";
        //             demojs.src = "app/bode/demo_transfer_function_bode.js";
                    
        //             demojs.onload = function() {
        //                 setup();
        //             };
        //             document.body.appendChild(demojs);
        //         };
        //         document.body.appendChild(controlplotjs);
        //     };
        //     document.body.appendChild(highchartsjs);
        // };
        // document.body.appendChild(hcframeworkjs);
        
        var tooltip = document.createElement("span");
        tooltip.className = "tooltiptext";
        tooltip.innerText = "WARNING: Please make sure External Clock Configuration is selected to match the actual external clock before enabling EXT_CLK_EN. A mismatch could result in device reset";
        document.getElementById("ti_adv_tuning_mdbu_custom_toggle_component3").appendChild(tooltip);
        
        function readAppReset() {
          var clk_sel = gc.databind.registry.getBinding('my_model.PERI_CONFIG1.CLK_SEL').getValue();
          var ext_clk_en = gc.databind.registry.getBinding('my_model.DEVICE_CONFIG1.EXT_CLK_EN').getValue();;
          if (clk_sel == 3 && ext_clk_en == 1 && gc.connectionManager.status == "connected" && !isNVMInProgress) {
            console.log("readAppReset: connected");
            var obj = {
              "addr": 226,
              "size": 32
            };
            CustomCodec.prototype.readValue(obj)
              .then(function (regValue) {
                var regReadCmdValue = 0x00000001 & regValue;
                var regWriteCmd = regValue & ~(0x00000001);
                if (regReadCmdValue)
                {
                    CustomCodec.prototype.writeValue(obj, regWriteCmd);
                    let registerURIs = ['my_model.DEVICE_CONFIG1',
                                        'my_model.PERI_CONFIG1'
                    ];
                    readRegisters(registerURIs);
                }
              }, function (err) {
                templateObj.showToastMsg('error', "App Reset Read failed", 4);
              });
          }
        }
        
        setInterval(readAppReset, 500);
    }, 1);

  });
};

templateObj = document.querySelector('#template_obj');
if (templateObj) {
  init();
} else {
  document.addEventListener('DOMContentLoaded', init);
}

var readFaultRegister = function () {
  if (gc.connectionManager.status == "connected") {
    let registerURIs = ['my_model.FAULT_STATUS1'];
    readRegisters(registerURIs).then(() => {
      return true;
    }).delay(100).then(() => {
      readFaultRegister();
    });
  }
}

function initializeSpeedProfilePlot() {
    // Initialize speed profile parameters to placeholder values
    speed_profile_params = {
        plot_type: 0,
        
        duty_off1: 0,
        duty_on1: 10,
        duty_clamp1: 20,
        duty_a: 35,
        duty_b: 45,
        duty_c: 55,
        duty_d: 65,
        duty_e: 75,
        duty_clamp2: 80,
        duty_on2: 90,
        duty_off2: 98,
        
        speed_off1: 0,
        speed_clamp1: 15,
        speed_a: 35,
        speed_b: 45,
        speed_c: 50,
        speed_d: 65,
        speed_e: 75,
        speed_clamp2: 100,
        speed_off2: 10
    };
    
    // Define speed profile axis
    linear_duty = [0,speed_profile_params.duty_off1,speed_profile_params.duty_off1,speed_profile_params.duty_on1,speed_profile_params.duty_clamp1,speed_profile_params.duty_a,speed_profile_params.duty_b,speed_profile_params.duty_c,speed_profile_params.duty_d,speed_profile_params.duty_e,speed_profile_params.duty_clamp2,speed_profile_params.duty_on2,speed_profile_params.duty_off2,speed_profile_params.duty_off2,100];
    linear_speed = [speed_profile_params.speed_off1,speed_profile_params.speed_off1,speed_profile_params.speed_clamp1,speed_profile_params.speed_clamp1,speed_profile_params.speed_clamp1,speed_profile_params.speed_a,speed_profile_params.speed_b,speed_profile_params.speed_c,speed_profile_params.speed_d,speed_profile_params.speed_e,speed_profile_params.speed_clamp2,speed_profile_params.speed_clamp2,speed_profile_params.speed_clamp2,speed_profile_params.speed_off2,speed_profile_params.speed_off2];
      
    staircase_duty = [0,speed_profile_params.duty_off1,speed_profile_params.duty_off1,speed_profile_params.duty_on1,speed_profile_params.duty_clamp1,speed_profile_params.duty_clamp1,speed_profile_params.duty_a,speed_profile_params.duty_a,speed_profile_params.duty_b,speed_profile_params.duty_b,speed_profile_params.duty_c,speed_profile_params.duty_c,speed_profile_params.duty_d,speed_profile_params.duty_d,speed_profile_params.duty_e,speed_profile_params.duty_e,speed_profile_params.duty_clamp2,speed_profile_params.duty_on2,speed_profile_params.duty_off2,speed_profile_params.duty_off2,100];
    staircase_speed = [speed_profile_params.speed_off1,speed_profile_params.speed_off1,speed_profile_params.speed_clamp1,speed_profile_params.speed_clamp1,speed_profile_params.speed_clamp1,speed_profile_params.speed_a,speed_profile_params.speed_a,speed_profile_params.speed_b,speed_profile_params.speed_b,speed_profile_params.speed_c,speed_profile_params.speed_c,speed_profile_params.speed_d,speed_profile_params.speed_d,speed_profile_params.speed_e,speed_profile_params.speed_e,speed_profile_params.speed_clamp2,speed_profile_params.speed_clamp2,speed_profile_params.speed_clamp2,speed_profile_params.speed_clamp2,speed_profile_params.speed_off2,speed_profile_params.speed_off2];
      
    forward_reverse_duty = [0,speed_profile_params.duty_off1,speed_profile_params.duty_off1,speed_profile_params.duty_on1,speed_profile_params.duty_clamp1,speed_profile_params.duty_a,speed_profile_params.duty_b,speed_profile_params.duty_b,speed_profile_params.duty_c,speed_profile_params.duty_d,speed_profile_params.duty_d,speed_profile_params.duty_e,speed_profile_params.duty_clamp2,speed_profile_params.duty_on2,speed_profile_params.duty_off2,speed_profile_params.duty_off2,100];
    forward_reverse_speed = [speed_profile_params.speed_off1,speed_profile_params.speed_off1,speed_profile_params.speed_clamp1,speed_profile_params.speed_clamp1,speed_profile_params.speed_clamp1,speed_profile_params.speed_a,speed_profile_params.speed_a,0,0,0,speed_profile_params.speed_d,speed_profile_params.speed_d,speed_profile_params.speed_clamp2,speed_profile_params.speed_clamp2,speed_profile_params.speed_clamp2,speed_profile_params.speed_off2,speed_profile_params.speed_off2];

    // Define the linear and staircase plot layout
    linear_and_stair_layout = {
        autosize: false,
        width: 600,
        height: 600,
        xaxis: {
            tickmode: "array",
            tickvals: [speed_profile_params.duty_off1,speed_profile_params.duty_on1,speed_profile_params.duty_clamp1,speed_profile_params.duty_a,speed_profile_params.duty_b,speed_profile_params.duty_c,speed_profile_params.duty_d,speed_profile_params.duty_e,speed_profile_params.duty_clamp2,speed_profile_params.duty_on2,speed_profile_params.duty_off2],
            ticktext: ['duty_off1','duty_on1','duty_clamp1','duty_a','duty_b','duty_c','duty_d','duty_e','duty_clamp2','duty_on2','duty_off2'],
            automargin: true
        },
        yaxis: {
            tickmode: "array",
            tickvals: [speed_profile_params.speed_off1,speed_profile_params.speed_clamp1,speed_profile_params.speed_a,speed_profile_params.speed_b,speed_profile_params.speed_c,speed_profile_params.speed_d,speed_profile_params.speed_e,speed_profile_params.speed_clamp2,speed_profile_params.speed_off2],
            ticktext: ['speed_off1','speed_clamp1','speed_a','speed_b','speed_c','speed_d','speed_e','speed_clamp2','speed_off2'],
            automargin: true
        }
    }
    
    // Define the forward reverse plot layout
    forward_reverse_layout = {
        autosize: false,
        width: 600,
        height: 600,
        xaxis: {
            tickmode: "array",
            tickvals: [speed_profile_params.duty_off1,speed_profile_params.duty_on1,speed_profile_params.duty_clamp1,speed_profile_params.duty_a,speed_profile_params.duty_b,speed_profile_params.duty_c,speed_profile_params.duty_d,speed_profile_params.duty_e,speed_profile_params.duty_clamp2,speed_profile_params.duty_on2,speed_profile_params.duty_off2],
            ticktext: ['duty_off1','duty_on1','duty_clamp1','duty_a','duty_b','duty_c','duty_d','duty_e','duty_clamp2','duty_on2','duty_off2'],
            automargin: true
        },
        yaxis: {
            tickmode: "array",
            tickvals: [speed_profile_params.speed_off1,speed_profile_params.speed_clamp1,speed_profile_params.speed_a,speed_profile_params.speed_d,speed_profile_params.speed_clamp2,speed_profile_params.speed_off2],
            ticktext: ['speed_off1','speed_clamp1','speed_a','speed_d','speed_clamp2','speed_off2'],
            automargin: true
        }
    }
    
    // Draw plot (default: linear)
    templateObj.$.ti_widget_plot.data[0].x=linear_duty;
    templateObj.$.ti_widget_plot.data[0].y=linear_speed;
    templateObj.$.ti_widget_plot.layout = linear_and_stair_layout;
    
    templateObj.$.ti_widget_plot.redraw();
    
}

function updatePlotDuty() {
    // Update x axis values and layout for all plot types
    linear_duty = [0,speed_profile_params.duty_off1,speed_profile_params.duty_off1,speed_profile_params.duty_on1,speed_profile_params.duty_clamp1,speed_profile_params.duty_a,speed_profile_params.duty_b,speed_profile_params.duty_c,speed_profile_params.duty_d,speed_profile_params.duty_e,speed_profile_params.duty_clamp2,speed_profile_params.duty_on2,speed_profile_params.duty_off2,speed_profile_params.duty_off2,100];
    staircase_duty = [0,speed_profile_params.duty_off1,speed_profile_params.duty_off1,speed_profile_params.duty_on1,speed_profile_params.duty_clamp1,speed_profile_params.duty_clamp1,speed_profile_params.duty_a,speed_profile_params.duty_a,speed_profile_params.duty_b,speed_profile_params.duty_b,speed_profile_params.duty_c,speed_profile_params.duty_c,speed_profile_params.duty_d,speed_profile_params.duty_d,speed_profile_params.duty_e,speed_profile_params.duty_e,speed_profile_params.duty_clamp2,speed_profile_params.duty_on2,speed_profile_params.duty_off2,speed_profile_params.duty_off2,100];
    forward_reverse_duty = [0,speed_profile_params.duty_off1,speed_profile_params.duty_off1,speed_profile_params.duty_on1,speed_profile_params.duty_clamp1,speed_profile_params.duty_a,speed_profile_params.duty_b,speed_profile_params.duty_b,speed_profile_params.duty_c,speed_profile_params.duty_d,speed_profile_params.duty_d,speed_profile_params.duty_e,speed_profile_params.duty_clamp2,speed_profile_params.duty_on2,speed_profile_params.duty_off2,speed_profile_params.duty_off2,100];

    linear_and_stair_layout.xaxis.tickvals = [speed_profile_params.duty_off1,speed_profile_params.duty_on1,speed_profile_params.duty_clamp1,speed_profile_params.duty_a,speed_profile_params.duty_b,speed_profile_params.duty_c,speed_profile_params.duty_d,speed_profile_params.duty_e,speed_profile_params.duty_clamp2,speed_profile_params.duty_on2,speed_profile_params.duty_off2];
    forward_reverse_layout.xaxis.tickvals = [speed_profile_params.duty_off1,speed_profile_params.duty_on1,speed_profile_params.duty_clamp1,speed_profile_params.duty_a,speed_profile_params.duty_b,speed_profile_params.duty_c,speed_profile_params.duty_d,speed_profile_params.duty_e,speed_profile_params.duty_clamp2,speed_profile_params.duty_on2,speed_profile_params.duty_off2];

    // Set correct x axis and layout based on plot type
    if (speed_profile_params.plot_type == 0) {
        templateObj.$.ti_widget_plot.data[0].x = linear_duty;
        templateObj.$.ti_widget_plot.layout = linear_and_stair_layout;
    }
    else if (speed_profile_params.plot_type == 1) {
        templateObj.$.ti_widget_plot.data[0].x = staircase_duty;
        templateObj.$.ti_widget_plot.layout = linear_and_stair_layout;
    }
    else
    {
        templateObj.$.ti_widget_plot.data[0].x = forward_reverse_duty;
        templateObj.$.ti_widget_plot.layout = forward_reverse_layout;
    }
    
    // Redraw plot with updated values
    templateObj.$.ti_widget_plot.relayout();
    templateObj.$.ti_widget_plot.redraw();
}

function updatePlotSpeed() {
    // Update y axis values and layout for all plot types
    linear_speed = [speed_profile_params.speed_off1,speed_profile_params.speed_off1,speed_profile_params.speed_clamp1,speed_profile_params.speed_clamp1,speed_profile_params.speed_clamp1,speed_profile_params.speed_a,speed_profile_params.speed_b,speed_profile_params.speed_c,speed_profile_params.speed_d,speed_profile_params.speed_e,speed_profile_params.speed_clamp2,speed_profile_params.speed_clamp2,speed_profile_params.speed_clamp2,speed_profile_params.speed_off2,speed_profile_params.speed_off2];
    staircase_speed = [speed_profile_params.speed_off1,speed_profile_params.speed_off1,speed_profile_params.speed_clamp1,speed_profile_params.speed_clamp1,speed_profile_params.speed_clamp1,speed_profile_params.speed_a,speed_profile_params.speed_a,speed_profile_params.speed_b,speed_profile_params.speed_b,speed_profile_params.speed_c,speed_profile_params.speed_c,speed_profile_params.speed_d,speed_profile_params.speed_d,speed_profile_params.speed_e,speed_profile_params.speed_e,speed_profile_params.speed_clamp2,speed_profile_params.speed_clamp2,speed_profile_params.speed_clamp2,speed_profile_params.speed_clamp2,speed_profile_params.speed_off2,speed_profile_params.speed_off2];
    forward_reverse_speed = [speed_profile_params.speed_off1,speed_profile_params.speed_off1,speed_profile_params.speed_clamp1,speed_profile_params.speed_clamp1,speed_profile_params.speed_clamp1,speed_profile_params.speed_a,speed_profile_params.speed_a,0,0,0,speed_profile_params.speed_d,speed_profile_params.speed_d,speed_profile_params.speed_clamp2,speed_profile_params.speed_clamp2,speed_profile_params.speed_clamp2,speed_profile_params.speed_off2,speed_profile_params.speed_off2];

    linear_and_stair_layout.yaxis.tickvals = [speed_profile_params.speed_off1,speed_profile_params.speed_clamp1,speed_profile_params.speed_a,speed_profile_params.speed_b,speed_profile_params.speed_c,speed_profile_params.speed_d,speed_profile_params.speed_e,speed_profile_params.speed_clamp2,speed_profile_params.speed_off2];
    forward_reverse_layout.yaxis.tickvals = [speed_profile_params.speed_off1,speed_profile_params.speed_clamp1,speed_profile_params.speed_a,speed_profile_params.speed_d,speed_profile_params.speed_clamp2,speed_profile_params.speed_off2];

    // Set correct y axis and layout based on plot type
    if (speed_profile_params.plot_type == 0) {
        templateObj.$.ti_widget_plot.data[0].y = linear_speed;
        templateObj.$.ti_widget_plot.layout = linear_and_stair_layout;
    }
    else if (speed_profile_params.plot_type == 1) {
        templateObj.$.ti_widget_plot.data[0].y = staircase_speed;
        templateObj.$.ti_widget_plot.layout = linear_and_stair_layout;
    }
    else
    {
        templateObj.$.ti_widget_plot.data[0].y = forward_reverse_speed;
        templateObj.$.ti_widget_plot.layout = forward_reverse_layout;
    }
    
    // Redraw plot with updated values
    templateObj.$.ti_widget_plot.relayout();
    templateObj.$.ti_widget_plot.redraw();
}
