customFieldsGetterGlobalData = {
  regBlocks: []
}

function fetchFieldValues(fieldsData) {
  let fieldsArrayData = [];
  let resultData = {};
  let keys = Object.keys(fieldsData)
  for (let key of keys) {
    let data = fieldsData[key];
    let result = {
      name: key,
      fieldData: undefined,
      registerValue: undefined
    }
    for (let regBlock of customFieldsGetterGlobalData.regBlocks) {
      for (let register of regBlock.registers) {
        if (register.name === data.split('.')[0]) {
          for (let field of register.fields) {
            if (field.name === data.split('.')[1]) {
              result.fieldData = field;
            }
          }
        }
      }
    }
    fieldsArrayData.push(result);
  }

  var prom = Q.when(0);

  fieldsArrayData.forEach((fieldArrayData) => {
    prom = prom.then(function () {
      if (fieldArrayData.registerValue === undefined) {
        let param = {
          "addr": fieldArrayData.fieldData.parentRegister.addr,
          "name": fieldArrayData.fieldData.parentRegister.name,
          "size": 32
        }
        return customReadRegister(param).then(value => {
          fieldArrayData.registerValue = value;
          resultData[fieldArrayData.name] = getFieldValueFromRegisterValue(fieldArrayData.registerValue, fieldArrayData.fieldData.start, fieldArrayData.fieldData.stop);
          for (let arrayData of fieldsArrayData) {
            if (arrayData.fieldData.parentRegister.name === fieldArrayData.fieldData.parentRegister.name) {
              arrayData.registerValue = fieldArrayData.registerValue;
              resultData[arrayData.name] = getFieldValueFromRegisterValue(arrayData.registerValue, arrayData.fieldData.start, arrayData.fieldData.stop);
            }
          }
          return resultData;
        })
      }
      return resultData;
    })
  })
  return prom;
}

function getFieldValueFromRegisterValue(registerValue, startBit, stopBit) {
  let fieldBinaryValue = registerValue.toString(2).split('').reverse().join('').slice(parseInt(startBit), parseInt(stopBit) + 1)
    .split('').reverse().join().replace(/,/g, '');
  return parseInt(fieldBinaryValue, 2);
}

function customReadRegister(param) {
  return CustomCodec.prototype.readValue(param);
}

function _hex2bin(hex, length) {
  let zeroes = function () {
    let result = '';
    for (let i = 0; i < length; i++) {
      result += '0';
    }
    return result;
  }();
  return (zeroes + (parseInt(hex, 16)).toString(2)).substr(-length);
}

function readCustomField(customFunction) {
  let deferred = Q.defer();
  window[customFunction]().then((value) => {
    deferred.resolve(value);
  }).catch((reason) => {
    console.log(reason);
    deferred.reject(reason);
  })
  return deferred.promise;
}

function getDroplistValues(droplistId) {
  var patt = /[+-]?\d+(\.\d+)?/g;
  if (document.querySelector('#' + droplistId)) {
    return document.querySelector('#' + droplistId).droplistLabels
      .split('|')
      .map((item, index) => {
        return item.match(patt) ? item.match(patt)[0] : index;
      });
  }
  return null;
}

function scrollTo(element, toElement, duration=600) {
  var start = element.scrollTop,
    change = toElement.offsetTop - start,
    currentTime = 0,
    increment = 20;

  let easeInOutQuad = function (t, b, c, d) {
    t /= d / 2;
    if (t < 1) return c / 2 * t * t + b;
    t--;
    return -c / 2 * (t * (t - 2) - 1) + b;
  };

  let animateScroll = function () {
    currentTime += increment;
    var val = easeInOutQuad(currentTime, start, change, duration);
    element.scrollTop = val;
    if (currentTime < duration) {
      setTimeout(animateScroll, increment);
    }
  };
  animateScroll();
}

function readRegisters(registerURIs) {
  let deferred = Q.defer();
  let processedRegistersCount = 0;
  let totalRegistersCount = registerURIs.length;
  for(let registerURI of registerURIs) {
    readRegister(registerURI).then(() => {
      processedRegistersCount++;
      if(processedRegistersCount === totalRegistersCount) {
        deferred.resolve();
      }
    });
  }
  return deferred.promise;
}

function readRegister(registerURI) {
  const deferred = Q.defer();
  const registerBind = this._createBindingCollection(registerURI);
  const progressMonitor = new gc.databind.ProgressCounter(function() {
    const result = registerBind.getValue();
    deferred.resolve(result);
  });
  if (registerBind.onRefresh) {
    registerBind.onRefresh(progressMonitor);
  } else {
    const errorMsg =
      "readValueFromTargetRegister failed: no onRefresh method for " +
      registerURI;
    deferred.reject(errorMsg);
  }
  progressMonitor.done();
  return deferred.promise;
}

function _createBindingCollection(bindings) {
  if (typeof bindings === "object") {
    const result = {};
    for (const bindName in bindings) {
      if (bindings.hasOwnProperty(bindName)) {
        let binding;
        try {
          // TODO: This gc.databind should be changed once
          // we have get binding from model interface
          binding = gc.databind.registry.getBinding(bindings[bindName]);
        } catch (e) {
          throw "Can't parse binding \"" + bindName + '".  \n' + e;
        }
        if (binding !== null) {
          result[bindName] = binding;
        } else {
          throw 'Binding "' + bindName + '" could not be found.';
        }
      }
    }
    return new gc.databind.CollectionBindValue(result);
  } else {
    try {
      return gc.databind.registry.getBinding(bindings);
    } catch (message) {
      throw "Can't parse binding \"" + bindings + '".  \n' + message;
    }
  }
}